/* *****************************************************************************
 *  Name:    Alan Turing
 *  NetID:   aturing
 *  Precept: P00
 *
 *  Partner Name:    Ada Lovelace
 *  Partner NetID:   alovelace
 *  Partner Precept: P00
 *
 *  Description:  A Bank that holds various unique accounts. Has basic
 *                functionality to add and get accounts and calculate the total
 *                summed balance across accounts.
 *
 **************************************************************************** */

import edu.princeton.cs.algs4.StdOut;

import java.util.LinkedList;

public class Bank {
    private final LinkedList<Account> accounts; // accounts held by bank

    // initialize empty bank
    public Bank() {
        accounts = new LinkedList<>();
    }

    // add account to bank, assumes accounts are unique instances
    public void addAccount(Account a) {
        accounts.add(a);
    }

    // return all accounts
    public Iterable<Account> getAccounts() {
        return accounts;
    }

    // calculate total sum across all accounts
    public int balanceAcrossAccounts() {
        int balance = 0;
        for (Account a : accounts) {
            balance += a.checkBalance();
        }
        return balance;
    }

    // calculate total sum across all accounts recursively
    public int balanceAcrossAccountsRecursive() {
        return balanceAcrossAccountsRecursive(0, 0);
    }

    // gets the sum of accounts from index i to end of list of accounts
    private int balanceAcrossAccountsRecursive(int i, int runningSum) {
        if (i < accounts.size()) {
            int ithAmount = accounts.get(i).checkBalance();
            return balanceAcrossAccountsRecursive(i + 1,
                                                  ithAmount + runningSum);
        }
        return runningSum;
    }

    // basic unit tests
    public static void main(String[] args) {
        Bank bank = new Bank();
        for (int i = 1; i < 5; i++) {
            Account a = new Account(i);
            bank.addAccount(a);
        }

        StdOut.println("expect 10: " + bank.balanceAcrossAccounts());
        StdOut.println("expect 10: " + bank.balanceAcrossAccountsRecursive());

        for (Account a : bank.getAccounts()) {
            a.withdraw(2);
        }

        StdOut.println("expect 2: " + bank.balanceAcrossAccounts());
        StdOut.println("expect 2: " + bank.balanceAcrossAccountsRecursive());
    }
}
